@@ -17,7 +17,7 @@ from nautobot.utilities.forms import (
 from nautobot.utilities.forms.forms import DynamicFilterFormSet
 from nautobot.utilities.paginator import EnhancedPaginator, get_paginate_count
 from nautobot.utilities.permissions import get_permission_for_model
-from nautobot.utilities.templatetags.helpers import validated_viewname
+from nautobot.utilities.templatetags.helpers import bettertitle, validated_viewname
 from nautobot.utilities.utils import (
     convert_querydict_to_factory_formset_acceptable_querydict,
     normalize_querydict,
@@ -246,6 +246,8 @@ class NautobotHTMLRenderer(renderers.BrowsableAPIRenderer):
                 context.update(
                     {
                         "action_buttons": valid_actions,
+                        "list_url": validated_viewname(model, "list"),
+                        "title": bettertitle(model._meta.verbose_name_plural),
                     }
                 )
             elif view.action in ["create", "update"]:
