@@ -1,14 +1,17 @@
 from django.contrib.contenttypes.models import ContentType
-from django.urls import reverse
 from django.test import override_settings
+from django.urls import reverse
+from django.utils.html import escape
 from rest_framework import status
 
 from nautobot.core.graphql import execute_query
+from nautobot.dcim.choices import InterfaceModeChoices
 from nautobot.dcim.models import Site
+from nautobot.extras import context_managers
 from nautobot.extras.choices import CustomFieldTypeChoices, ObjectChangeActionChoices, ObjectChangeEventContextChoices
 from nautobot.extras.models import CustomField, CustomFieldChoice, ObjectChange, Status, Tag
 from nautobot.ipam.models import VLAN
-from nautobot.utilities.testing import APITestCase
+from nautobot.utilities.testing import APITestCase, TestCase
 from nautobot.utilities.testing.utils import post_data
 from nautobot.utilities.testing.views import ModelViewTestCase
 from nautobot.utilities.utils import get_changes_for_model
@@ -39,15 +42,18 @@ class ChangeLogViewTest(ModelViewTestCase):
         CustomFieldChoice.objects.create(field=cf_select, value="Bar")
         CustomFieldChoice.objects.create(field=cf_select, value="Foo")
 
+        cls.tags = Tag.objects.get_for_model(Site)
+
+        cls.site_status = Status.objects.get_for_model(Site).first()
+
     def test_create_object(self):
-        tags = self.create_tags("Tag 1", "Tag 2")
         form_data = {
             "name": "Test Site 1",
             "slug": "test-site-1",
             "status": Status.objects.get(slug="active").pk,
             "cf_my_field": "ABC",
             "cf_my_field_select": "Bar",
-            "tags": [tag.pk for tag in tags],
+            "tags": [tag.pk for tag in self.tags],
         }
 
         request = {
@@ -66,7 +72,7 @@ class ChangeLogViewTest(ModelViewTestCase):
         self.assertEqual(oc.action, ObjectChangeActionChoices.ACTION_CREATE)
         self.assertEqual(oc.object_data["custom_fields"]["my_field"], form_data["cf_my_field"])
         self.assertEqual(oc.object_data["custom_fields"]["my_field_select"], form_data["cf_my_field_select"])
-        self.assertEqual(oc.object_data["tags"], ["Tag 1", "Tag 2"])
+        self.assertEqual(oc.object_data["tags"], sorted([tag.name for tag in self.tags]))
         self.assertEqual(oc.user_id, self.user.pk)
 
     def test_update_object(self):
@@ -76,8 +82,7 @@ class ChangeLogViewTest(ModelViewTestCase):
             status=Status.objects.get(slug="active"),
         )
         site.save()
-        tags = self.create_tags("Tag 1", "Tag 2", "Tag 3")
-        site.tags.set("Tag 1", "Tag 2")
+        site.tags.set(self.tags[:2])
 
         form_data = {
             "name": "Test Site X",
@@ -85,7 +90,7 @@ class ChangeLogViewTest(ModelViewTestCase):
             "status": Status.objects.get(slug="planned").pk,
             "cf_my_field": "DEF",
             "cf_my_field_select": "Foo",
-            "tags": [tags[2].pk],
+            "tags": [self.tags[2].pk],
         }
 
         request = {
@@ -106,7 +111,7 @@ class ChangeLogViewTest(ModelViewTestCase):
             oc.object_data["custom_fields"]["my_field_select"],
             form_data["cf_my_field_select"],
         )
-        self.assertEqual(oc.object_data["tags"], ["Tag 3"])
+        self.assertEqual(oc.object_data["tags"], [self.tags[2].name])
         self.assertEqual(oc.user_id, self.user.pk)
 
     def test_delete_object(self):
@@ -116,8 +121,7 @@ class ChangeLogViewTest(ModelViewTestCase):
             _custom_field_data={"my_field": "ABC", "my_field_select": "Bar"},
         )
         site.save()
-        self.create_tags("Tag 1", "Tag 2")
-        site.tags.set("Tag 1", "Tag 2")
+        site.tags.set(self.tags)
 
         request = {
             "path": self._get_url("delete", instance=site),
@@ -133,7 +137,7 @@ class ChangeLogViewTest(ModelViewTestCase):
         self.assertEqual(oc.action, ObjectChangeActionChoices.ACTION_DELETE)
         self.assertEqual(oc.object_data["custom_fields"]["my_field"], "ABC")
         self.assertEqual(oc.object_data["custom_fields"]["my_field_select"], "Bar")
-        self.assertEqual(oc.object_data["tags"], ["Tag 1", "Tag 2"])
+        self.assertEqual(oc.object_data["tags"], sorted([tag.name for tag in self.tags]))
         self.assertEqual(oc.user_id, self.user.pk)
 
     def test_change_context(self):
@@ -158,6 +162,47 @@ class ChangeLogViewTest(ModelViewTestCase):
         self.assertEqual(oc.change_context_detail, "dcim:site_add")
         self.assertEqual(oc.user_id, self.user.pk)
 
+    def test_legacy_object_data(self):
+        self.add_permissions("dcim.view_site", "extras.view_objectchange")
+        with context_managers.web_request_context(self.user):
+            site = Site.objects.create(
+                name="testobjectchangesite", description="initial description", status=self.site_status
+            )
+
+        # create objectchange without object_data_v2
+        with context_managers.web_request_context(self.user):
+            site.description = "changed description1"
+            site.validated_save()
+        oc_without_object_data_v2_1 = get_changes_for_model(site).first()
+        oc_without_object_data_v2_1.object_data_v2 = None
+        oc_without_object_data_v2_1.validated_save()
+        with self.subTest("previous ObjectChange has object_data_v2, current ObjectChange does not"):
+            resp = self.client.get(oc_without_object_data_v2_1.get_absolute_url())
+            self.assertContains(resp, escape('"description": "initial description"'))
+            self.assertContains(resp, escape('"description": "changed description1"'))
+
+        # create second objectchange without object_data_v2
+        with context_managers.web_request_context(self.user):
+            site.description = "changed description2"
+            site.validated_save()
+        oc_without_object_data_v2_2 = get_changes_for_model(site).first()
+        oc_without_object_data_v2_2.object_data_v2 = None
+        oc_without_object_data_v2_2.validated_save()
+        with self.subTest("previous and current ObjectChange do not have object_data_v2"):
+            resp = self.client.get(oc_without_object_data_v2_2.get_absolute_url())
+            self.assertContains(resp, escape('"description": "changed description1"'))
+            self.assertContains(resp, escape('"description": "changed description2"'))
+
+        # create objectchange with object_data_v2
+        with context_managers.web_request_context(self.user):
+            site.description = "changed description3"
+            site.validated_save()
+        oc_with_object_data_v2 = get_changes_for_model(site).first()
+        with self.subTest("previous ObjectChange does not have object_data_v2, current ObjectChange does"):
+            resp = self.client.get(oc_with_object_data_v2.get_absolute_url())
+            self.assertContains(resp, escape('"description": "changed description2"'))
+            self.assertContains(resp, escape('"description": "changed description3"'))
+
 
 class ChangeLogAPITest(APITestCase):
     def setUp(self):
@@ -181,15 +226,7 @@ class ChangeLogAPITest(APITestCase):
         CustomFieldChoice.objects.create(field=cf_select, value="Bar")
         CustomFieldChoice.objects.create(field=cf_select, value="Foo")
 
-        # Create some tags
-        tags = (
-            Tag.objects.create(name="Tag 1", slug="tag-1"),
-            Tag.objects.create(name="Tag 2", slug="tag-2"),
-            Tag.objects.create(name="Tag 3", slug="tag-3"),
-        )
-        for tag in tags:
-            tag.content_types.add(ContentType.objects.get_for_model(Site))
-
+        self.tags = Tag.objects.get_for_model(Site)
         self.statuses = Status.objects.get_for_model(Site)
 
     def test_create_object(self):
@@ -202,8 +239,8 @@ class ChangeLogAPITest(APITestCase):
                 "my_field_select": "Bar",
             },
             "tags": [
-                {"name": "Tag 1"},
-                {"name": "Tag 2"},
+                {"name": self.tags[0].name},
+                {"name": self.tags[1].name},
             ],
         }
         self.assertEqual(ObjectChange.objects.count(), 0)
@@ -218,7 +255,7 @@ class ChangeLogAPITest(APITestCase):
         self.assertEqual(oc.changed_object, site)
         self.assertEqual(oc.action, ObjectChangeActionChoices.ACTION_CREATE)
         self.assertEqual(oc.object_data["custom_fields"], data["custom_fields"])
-        self.assertEqual(oc.object_data["tags"], ["Tag 1", "Tag 2"])
+        self.assertEqual(oc.object_data["tags"], sorted([self.tags[0].name, self.tags[1].name]))
         self.assertEqual(oc.user_id, self.user.pk)
 
     def test_update_object(self):
@@ -237,7 +274,7 @@ class ChangeLogAPITest(APITestCase):
                 "my_field": "DEF",
                 "my_field_select": "Foo",
             },
-            "tags": [{"name": "Tag 3"}],
+            "tags": [{"name": self.tags[2].name}],
         }
         self.assertEqual(ObjectChange.objects.count(), 0)
         self.add_permissions("dcim.change_site", "extras.view_status")
@@ -251,7 +288,7 @@ class ChangeLogAPITest(APITestCase):
         self.assertEqual(oc.changed_object, site)
         self.assertEqual(oc.action, ObjectChangeActionChoices.ACTION_UPDATE)
         self.assertEqual(oc.object_data["custom_fields"], data["custom_fields"])
-        self.assertEqual(oc.object_data["tags"], ["Tag 3"])
+        self.assertEqual(oc.object_data["tags"], [self.tags[2].name])
         self.assertEqual(oc.user_id, self.user.pk)
 
     def test_partial_update_object(self):
@@ -265,7 +302,7 @@ class ChangeLogAPITest(APITestCase):
                 "my_field_select": "Foo",
             },
         )
-        site.tags.add(Tag.objects.get(name="Tag 3"))
+        site.tags.add(self.tags[2])
 
         # We only want to update a single field.
         data = {
@@ -287,7 +324,7 @@ class ChangeLogAPITest(APITestCase):
         self.assertEqual(oc.object_data["description"], data["description"])
         self.assertEqual(oc.action, ObjectChangeActionChoices.ACTION_UPDATE)
         self.assertEqual(oc.object_data["custom_fields"], site.custom_field_data)
-        self.assertEqual(oc.object_data["tags"], ["Tag 3"])
+        self.assertEqual(oc.object_data["tags"], [self.tags[2].name])
         self.assertEqual(oc.user_id, self.user.pk)
 
     def test_delete_object(self):
@@ -298,14 +335,15 @@ class ChangeLogAPITest(APITestCase):
             _custom_field_data={"my_field": "ABC", "my_field_select": "Bar"},
         )
         site.save()
-        site.tags.set(*Tag.objects.all()[:2])
+        site.tags.set(self.tags[:2])
         self.assertEqual(ObjectChange.objects.count(), 0)
         self.add_permissions("dcim.delete_site", "extras.view_status")
         url = reverse("dcim-api:site-detail", kwargs={"pk": site.pk})
+        initial_count = Site.objects.count()
 
         response = self.client.delete(url, **self.header)
         self.assertHttpStatus(response, status.HTTP_204_NO_CONTENT)
-        self.assertEqual(Site.objects.count(), 0)
+        self.assertEqual(Site.objects.count(), initial_count - 1)
 
         oc = ObjectChange.objects.first()
         self.assertEqual(oc.changed_object, None)
@@ -313,7 +351,7 @@ class ChangeLogAPITest(APITestCase):
         self.assertEqual(oc.action, ObjectChangeActionChoices.ACTION_DELETE)
         self.assertEqual(oc.object_data["custom_fields"]["my_field"], "ABC")
         self.assertEqual(oc.object_data["custom_fields"]["my_field_select"], "Bar")
-        self.assertEqual(oc.object_data["tags"], ["Tag 1", "Tag 2"])
+        self.assertEqual(oc.object_data["tags"], sorted([tag.name for tag in self.tags[:2]]))
         self.assertEqual(oc.user_id, self.user.pk)
 
     @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
@@ -409,6 +447,7 @@ class ChangeLogAPITest(APITestCase):
             name="vm interface 1",
             virtual_machine=vm,
             status=vminterface_statuses.get(slug="active"),
+            mode=InterfaceModeChoices.MODE_TAGGED,
         )
         vlan_statuses = Status.objects.get_for_model(VLAN)
         tagged_vlan = VLAN.objects.create(vid=100, name="Vlan100", status=vlan_statuses.get(slug="active"))
@@ -426,3 +465,77 @@ class ChangeLogAPITest(APITestCase):
         self.assertEqual(oc.user_id, self.user.pk)
         self.assertEqual(vm_interface.description, "test vm interface m2m change")
         self.assertSequenceEqual(list(vm_interface.tagged_vlans.all()), [tagged_vlan])
+
+
+class ObjectChangeModelTest(TestCase):
+    @classmethod
+    def setUpTestData(cls):
+        cls.site_status = Status.objects.get_for_model(Site).first()
+
+    def test_get_snapshots(self):
+        with context_managers.web_request_context(self.user):
+            site = Site.objects.create(
+                name="testobjectchangesite", description="initial description", status=self.site_status
+            )
+        initial_object_change = get_changes_for_model(site).first()
+
+        with self.subTest("test get_snapshots ObjectChange create"):
+            snapshots = initial_object_change.get_snapshots()
+            self.assertIsNone(snapshots["prechange"])
+            self.assertEqual(snapshots["postchange"], initial_object_change.object_data_v2)
+            self.assertIsNone(snapshots["differences"]["removed"])
+            self.assertEqual(snapshots["differences"]["added"], initial_object_change.object_data_v2)
+
+        # first objectchange without object_data_v2
+        with context_managers.web_request_context(self.user):
+            site.description = "changed description1"
+            site.validated_save()
+        oc_without_object_data_v2_1 = get_changes_for_model(site).first()
+        oc_without_object_data_v2_1.object_data_v2 = None
+        oc_without_object_data_v2_1.validated_save()
+        with self.subTest("test get_snapshots previous ObjectChange has object_data_v2, current ObjectChange does not"):
+            snapshots = oc_without_object_data_v2_1.get_snapshots()
+            self.assertEqual(snapshots["prechange"], initial_object_change.object_data)
+            self.assertEqual(snapshots["postchange"], oc_without_object_data_v2_1.object_data)
+            self.assertEqual(snapshots["differences"]["removed"], {"description": "initial description"})
+            self.assertEqual(snapshots["differences"]["added"], {"description": "changed description1"})
+
+        # second objectchange without object_data_v2
+        with context_managers.web_request_context(self.user):
+            site.description = "changed description2"
+            site.validated_save()
+        oc_without_object_data_v2_2 = get_changes_for_model(site).first()
+        oc_without_object_data_v2_2.object_data_v2 = None
+        oc_without_object_data_v2_2.validated_save()
+        with self.subTest("test get_snapshots previous and current ObjectChange do not have object_data_v2"):
+            snapshots = oc_without_object_data_v2_2.get_snapshots()
+            self.assertEqual(snapshots["prechange"], oc_without_object_data_v2_1.object_data)
+            self.assertEqual(snapshots["postchange"], oc_without_object_data_v2_2.object_data)
+            self.assertEqual(snapshots["differences"]["removed"], {"description": "changed description1"})
+            self.assertEqual(snapshots["differences"]["added"], {"description": "changed description2"})
+
+        # objectchange with object_data_v2
+        with context_managers.web_request_context(self.user):
+            site.description = "changed description3"
+            site.validated_save()
+        oc_with_object_data_v2 = get_changes_for_model(site).first()
+        with self.subTest(
+            "test get_snapshots previous ObjectChange does not have object_data_v2, current ObjectChange does"
+        ):
+            snapshots = oc_with_object_data_v2.get_snapshots()
+            self.assertEqual(snapshots["prechange"], oc_without_object_data_v2_2.object_data)
+            self.assertEqual(snapshots["postchange"], oc_with_object_data_v2.object_data)
+            self.assertEqual(snapshots["differences"]["removed"], {"description": "changed description2"})
+            self.assertEqual(snapshots["differences"]["added"], {"description": "changed description3"})
+
+        # objectchange action delete
+        site_pk = site.pk
+        with context_managers.web_request_context(self.user):
+            site.delete()
+        oc_delete = get_changes_for_model(Site).filter(changed_object_id=site_pk).first()
+        with self.subTest("test get_snapshots ObjectChange delete"):
+            snapshots = oc_delete.get_snapshots()
+            self.assertEqual(snapshots["prechange"], oc_with_object_data_v2.object_data_v2)
+            self.assertIsNone(snapshots["postchange"])
+            self.assertEqual(snapshots["differences"]["removed"], oc_with_object_data_v2.object_data_v2)
+            self.assertIsNone(snapshots["differences"]["added"])
