@@ -3,7 +3,6 @@ from datetime import timedelta
 import logging
 
 from celery import chain
-from django import template
 from django.contrib import messages
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ObjectDoesNotExist
@@ -26,12 +25,13 @@ from nautobot.core.views import generic
 from nautobot.dcim.models import Device
 from nautobot.dcim.tables import DeviceTable
 from nautobot.extras.tasks import delete_custom_field_data
-from nautobot.extras.utils import get_job_content_type, get_worker_count
+from nautobot.extras.utils import get_base_template, get_job_content_type, get_worker_count
 from nautobot.utilities.paginator import EnhancedPaginator, get_paginate_count
 from nautobot.utilities.forms import restrict_form_fields
 from nautobot.utilities.utils import (
     copy_safe_request,
     count_related,
+    csv_format,
     get_table_for_model,
     prepare_cloned_fields,
     pretty_print_query,
@@ -62,6 +62,7 @@ from .models import (
     ImageAttachment,
     Job as JobModel,
     JobHook,
+    JobLogEntry,
     ObjectChange,
     JobResult,
     Relationship,
@@ -118,7 +119,7 @@ class ComputedFieldBulkDeleteView(generic.BulkDeleteView):
 # Config contexts
 #
 
-# TODO: disallow (or at least warn) user from manually editing config contexts that
+# TODO(Glenn): disallow (or at least warn) user from manually editing config contexts that
 # have an associated owner, such as a Git repository
 
 
@@ -201,7 +202,7 @@ class ObjectConfigContextView(generic.ObjectView):
 # Config context schemas
 #
 
-# TODO: disallow (or at least warn) user from manually editing config context schemas that
+# TODO(Glenn): disallow (or at least warn) user from manually editing config context schemas that
 # have an associated owner, such as a Git repository
 
 
@@ -269,6 +270,7 @@ class ConfigContextSchemaObjectValidationView(generic.ObjectView):
 
         # Device table
         device_table = DeviceTable(
+            # v2 TODO(jathan): Replace prefetch_related with select_related
             data=instance.device_set.prefetch_related(
                 "tenant", "site", "rack", "device_type", "device_role", "primary_ip"
             ),
@@ -289,6 +291,7 @@ class ConfigContextSchemaObjectValidationView(generic.ObjectView):
 
         # Virtual machine table
         virtual_machine_table = VirtualMachineTable(
+            # v2 TODO(jathan): Replace prefetch_related with select_related
             data=instance.virtualmachine_set.prefetch_related("cluster", "role", "tenant", "primary_ip"),
             orderable=False,
             extra_columns=[
@@ -473,7 +476,7 @@ class CustomFieldBulkDeleteView(generic.BulkDeleteView):
         """
         Remove all Custom Field Keys/Values from _custom_field_data of the related ContentType in the background.
         """
-        if not get_worker_count(request):
+        if not get_worker_count():
             messages.error(
                 request, "Celery worker process not running. Object custom fields may fail to reflect this deletion."
             )
@@ -580,9 +583,6 @@ class DynamicGroupEditView(generic.ObjectEditView):
         filterform_class = instance.generate_filter_form()
 
         if filterform_class is None:
-            # FIXME(jathan): There is currently an edge case here that needs to be addressed:
-            # `AttributeError: 'NoneType' object has no attribute 'is_valid'`
-            # See: https://sentry.io/share/issue/fb41c6afb40248f6931021574bc38a0d/
             filter_form = None
         elif request.POST:
             filter_form = filterform_class(data=request.POST)
@@ -592,8 +592,6 @@ class DynamicGroupEditView(generic.ObjectEditView):
 
         ctx["filter_form"] = filter_form
 
-        # FIXME(jathan): Editing filter or groups are distinct. Both should not
-        # be possible. For now let's just get it working.
         formset_kwargs = {"instance": instance}
         if request.POST:
             formset_kwargs["data"] = request.POST
@@ -727,15 +725,7 @@ class ObjectDynamicGroupsView(View):
         }
         RequestConfig(request, paginate).configure(dynamicsgroups_table)
 
-        # Default to using "<app>/<model>.html" as the template, if it exists. Otherwise,
-        # fall back to using base.html.
-        if self.base_template is None:
-            self.base_template = f"{model._meta.app_label}/{model._meta.model_name}.html"
-            # TODO: This can be removed once an object view has been established for every model.
-            try:
-                template.loader.get_template(self.base_template)
-            except template.TemplateDoesNotExist:
-                self.base_template = "base.html"
+        self.base_template = get_base_template(self.base_template, model)
 
         return render(
             request,
@@ -856,6 +846,7 @@ class GitRepositoryBulkImportView(generic.BulkImportView):
 
 
 class GitRepositoryBulkEditView(generic.BulkEditView):
+    # v2 TODO(jathan): Replace prefetch_related with select_related
     queryset = GitRepository.objects.prefetch_related("secrets_group")
     filterset = filters.GitRepositoryFilterSet
     table = tables.GitRepositoryBulkTable
@@ -895,7 +886,7 @@ def check_and_call_git_repository_function(request, slug, func):
         return HttpResponseForbidden()
 
     # Allow execution only if a worker process is running.
-    if not get_worker_count(request):
+    if not get_worker_count():
         messages.error(request, "Unable to run job: Celery worker process not running.")
     else:
         repository = get_object_or_404(GitRepository, slug=slug)
@@ -1042,6 +1033,7 @@ class JobListView(generic.ObjectListView):
             queryset = queryset.filter(installed=True)
         if "is_job_hook_receiver" not in request.GET:
             queryset = queryset.filter(is_job_hook_receiver=False)
+        # v2 TODO(jathan): Replace prefetch_related with select_related
         queryset = queryset.prefetch_related("results")
         return queryset
 
@@ -1089,6 +1081,9 @@ class JobView(ObjectPermissionRequiredMixin, View):
                 commit = job_result.job_kwargs.get("commit")
                 if commit is not None:
                     initial.setdefault("_commit", commit)
+                task_queue = job_result.job_kwargs.get("task_queue")
+                if task_queue is not None:
+                    initial.setdefault("_task_queue", task_queue)
                 initial.update(explicit_initial)
             except JobResult.DoesNotExist:
                 messages.warning(request, f"JobResult {job_result_pk} not found, cannot use it to pre-populate inputs.")
@@ -1126,9 +1121,10 @@ class JobView(ObjectPermissionRequiredMixin, View):
             job_model.job_class().as_form(request.POST, request.FILES) if job_model.job_class is not None else None
         )
         schedule_form = forms.JobScheduleForm(request.POST)
+        task_queue = request.POST.get("_task_queue")
 
         # Allow execution only if a worker process is running and the job is runnable.
-        if not get_worker_count(request):
+        if not get_worker_count(queue=task_queue):
             messages.error(request, "Unable to run or schedule job: Celery worker process not running.")
         elif not job_model.installed or job_model.job_class is None:
             messages.error(request, "Unable to run or schedule job: Job is not presently installed.")
@@ -1180,7 +1176,10 @@ class JobView(ObjectPermissionRequiredMixin, View):
                     "user": request.user.pk,
                     "commit": commit,
                     "name": job_model.class_path,
+                    "task_queue": job_form.cleaned_data.get("_task_queue", None),
                 }
+                if task_queue:
+                    job_kwargs["celery_kwargs"] = {"queue": task_queue}
 
                 scheduled_job = ScheduledJob(
                     name=schedule_name,
@@ -1192,6 +1191,7 @@ class JobView(ObjectPermissionRequiredMixin, View):
                     kwargs=job_kwargs,
                     interval=schedule_type,
                     one_off=schedule_type == JobExecutionType.TYPE_FUTURE,
+                    queue=task_queue,
                     user=request.user,
                     approval_required=job_model.approval_required,
                     crontab=crontab,
@@ -1213,9 +1213,11 @@ class JobView(ObjectPermissionRequiredMixin, View):
                     job_model.class_path,
                     job_content_type,
                     request.user,
+                    celery_kwargs={"queue": task_queue},
                     data=job_model.job_class.serialize_data(job_form.cleaned_data),
                     request=copy_safe_request(request),
                     commit=commit,
+                    task_queue=job_form.cleaned_data.get("_task_queue", None),
                 )
 
                 return redirect("extras:jobresult", pk=job_result.pk)
@@ -1324,14 +1326,17 @@ class JobApprovalRequestView(generic.ObjectView):
                 job_content_type = get_job_content_type()
                 initial = scheduled_job.kwargs.get("data", {})
                 initial["_commit"] = False
+                celery_kwargs = scheduled_job.kwargs.get("celery_kwargs", {})
                 job_result = JobResult.enqueue_job(
                     run_job,
                     job_model.job_class.class_path,
                     job_content_type,
                     request.user,
+                    celery_kwargs=celery_kwargs,
                     data=job_model.job_class.serialize_data(initial),
                     request=copy_safe_request(request),
                     commit=False,  # force a dry-run
+                    task_queue=scheduled_job.kwargs.get("task_queue", None),
                 )
 
                 return redirect("extras:jobresult", pk=job_result.pk)
@@ -1472,6 +1477,7 @@ class JobResultListView(generic.ObjectListView):
     List JobResults
     """
 
+    # v2 TODO(jathan): Replace prefetch_related with select_related
     queryset = JobResult.objects.prefetch_related("job_model", "logs", "obj_type", "user")
     filterset = filters.JobResultFilterSet
     filterset_form = forms.JobResultFilterForm
@@ -1496,6 +1502,34 @@ class JobResultView(generic.ObjectView):
     queryset = JobResult.objects.prefetch_related("job_model", "obj_type", "user")
     template_name = "extras/jobresult.html"
 
+    def instance_to_csv(self, instance):
+        """Format instance to csv."""
+        csv_data = []
+        headers = JobLogEntry.csv_headers.copy()
+        csv_data.append(",".join(headers))
+
+        for log_entry in instance.logs.all():
+            data = log_entry.to_csv()
+            csv_data.append(csv_format(data))
+
+        return "\n".join(csv_data)
+
+    def get(self, request, *args, **kwargs):
+        """
+        Generic GET handler for accessing an object by PK or slug
+        """
+        instance = get_object_or_404(self.queryset, **kwargs)
+
+        if "export" in request.GET:
+            response = HttpResponse(self.instance_to_csv(instance), content_type="text/csv")
+            underscore_filename = f"{instance.job_model.slug.replace('-', '_')}"
+            formated_completion_time = instance.completed.strftime("%Y-%m-%d_%H_%M")
+            filename = f"{underscore_filename}_{formated_completion_time}_logs.csv"
+            response["Content-Disposition"] = f"attachment; filename={filename}"
+            return response
+
+        return super().get(request, *args, **kwargs)
+
     def get_extra_context(self, request, instance):
         associated_record = None
         job_class = None
@@ -1543,7 +1577,7 @@ class ObjectChangeListView(generic.ObjectListView):
     template_name = "extras/objectchange_list.html"
     action_buttons = ("export",)
 
-    # TODO: Remove this remapping in 2.0 as it is addressing a potentially breaking change
+    # 2.0 TODO: Remove this remapping and solve it at the `BaseFilterSet` as it is addressing a breaking change.
     def get(self, request, **kwargs):
 
         # Remappings below allow previous queries of time_before and time_after to use
@@ -1600,6 +1634,7 @@ class ObjectChangeLogView(View):
 
         # Gather all changes for this object (and its related objects)
         content_type = ContentType.objects.get_for_model(model)
+        # v2 TODO(jathan): Replace prefetch_related with select_related
         objectchanges = (
             ObjectChange.objects.restrict(request.user, "view")
             .prefetch_related("user", "changed_object_type")
@@ -1617,15 +1652,7 @@ class ObjectChangeLogView(View):
         }
         RequestConfig(request, paginate).configure(objectchanges_table)
 
-        # Default to using "<app>/<model>.html" as the template, if it exists. Otherwise,
-        # fall back to using base.html.
-        if self.base_template is None:
-            self.base_template = f"{model._meta.app_label}/{model._meta.model_name}.html"
-            # TODO: This can be removed once an object view has been established for every model.
-            try:
-                template.loader.get_template(self.base_template)
-            except template.TemplateDoesNotExist:
-                self.base_template = "base.html"
+        self.base_template = get_base_template(self.base_template, model)
 
         return render(
             request,
@@ -1694,15 +1721,7 @@ class ObjectNotesView(View):
         }
         RequestConfig(request, paginate).configure(notes_table)
 
-        # Default to using "<app>/<model>.html" as the template, if it exists. Otherwise,
-        # fall back to using base.html.
-        if self.base_template is None:
-            self.base_template = f"{model._meta.app_label}/{model._meta.model_name}.html"
-            # TODO: This can be removed once an object view has been established for every model.
-            try:
-                template.loader.get_template(self.base_template)
-            except template.TemplateDoesNotExist:
-                self.base_template = "base.html"
+        self.base_template = get_base_template(self.base_template, model)
 
         return render(
             request,
@@ -1714,7 +1733,7 @@ class ObjectNotesView(View):
                 "table": notes_table,
                 "base_template": self.base_template,
                 "active_tab": "notes",
-                "notes_form": notes_form,
+                "form": notes_form,
             },
         )
 
@@ -2040,6 +2059,7 @@ class TagView(generic.ObjectView):
     queryset = Tag.objects.all()
 
     def get_extra_context(self, request, instance):
+        # v2 TODO(jathan): Replace prefetch_related with select_related
         tagged_items = TaggedItem.objects.filter(tag=instance).prefetch_related("content_type", "content_object")
 
         # Generate a table of all items tagged with this Tag
@@ -2116,3 +2136,18 @@ class WebhookDeleteView(generic.ObjectDeleteView):
 class WebhookBulkDeleteView(generic.BulkDeleteView):
     queryset = Webhook.objects.all()
     table = tables.WebhookTable
+
+
+#
+# Job Extra Views
+#
+# NOTE: Due to inheritance, JobObjectChangeLogView and JobObjectNotesView can only be
+# constructed below # ObjectChangeLogView and ObjectNotesView.
+
+
+class JobObjectChangeLogView(ObjectChangeLogView):
+    base_template = "extras/job_detail.html"
+
+
+class JobObjectNotesView(ObjectNotesView):
+    base_template = "extras/job_detail.html"
