@@ -1,9 +1,11 @@
+from tree_queries.models import TreeNode
 from typing import Optional, Sequence
 import uuid
 
 from django.conf import settings
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ObjectDoesNotExist
+from django.core.files.uploadedfile import SimpleUploadedFile
 from django.test import TestCase as _TestCase, override_settings, tag
 from django.urls import reverse, NoReverseMatch
 from django.utils.text import slugify
@@ -11,9 +13,10 @@ from django.utils.text import slugify
 from nautobot.extras.choices import CustomFieldTypeChoices, RelationshipSideChoices, ObjectChangeActionChoices
 from nautobot.extras.models import ChangeLoggedModel, CustomField, Relationship
 from nautobot.users.models import ObjectPermission
+from nautobot.utilities.templatetags.helpers import bettertitle, validated_viewname
 from nautobot.utilities.testing.mixins import NautobotTestCaseMixin
 from nautobot.utilities.utils import get_changes_for_model, get_filterset_for_model
-from .utils import disable_warnings, extract_page_body, post_data
+from .utils import disable_warnings, extract_page_body, get_deletable_objects, post_data
 
 
 __all__ = (
@@ -25,6 +28,7 @@ __all__ = (
 
 
 @tag("unit")
+@override_settings(PAGINATE_COUNT=65000)
 class TestCase(_TestCase, NautobotTestCaseMixin):
     """Base class for all Nautobot-specific unit tests."""
 
@@ -58,6 +62,7 @@ class ModelTestCase(TestCase):
 #
 
 
+@tag("performance")
 class ModelViewTestCase(ModelTestCase):
     """
     Base TestCase for model views. Subclass to test individual views.
@@ -125,6 +130,10 @@ class ViewTestCases:
             self.client.logout()
             response = self.client.get(self._get_queryset().first().get_absolute_url())
             self.assertHttpStatus(response, 200)
+            response_body = response.content.decode(response.charset)
+            self.assertIn(
+                "/login/?next=" + self._get_queryset().first().get_absolute_url(), response_body, msg=response_body
+            )
 
             # The "Change Log" tab should appear in the response since we have all exempt permissions
             if issubclass(self.model, ChangeLoggedModel):
@@ -137,7 +146,10 @@ class ViewTestCases:
 
             # Try GET without permission
             with disable_warnings("django.request"):
-                self.assertHttpStatus(self.client.get(instance.get_absolute_url()), [403, 404])
+                response = self.client.get(instance.get_absolute_url())
+                self.assertHttpStatus(response, [403, 404])
+                response_body = response.content.decode(response.charset)
+                self.assertNotIn("/login/", response_body, msg=response_body)
 
         @override_settings(EXEMPT_VIEW_PERMISSIONS=[])
         def test_get_object_with_permission(self):
@@ -295,12 +307,18 @@ class ViewTestCases:
             }
             self.assertHttpStatus(self.client.post(**request), 302)
             self.assertEqual(initial_count + 1, self._get_queryset().count())
-            if hasattr(self.model, "last_updated"):
-                instance = self._get_queryset().order_by("last_updated").last()
+            # order_by() is no supported by django TreeNode,
+            # So we directly retrieve the instance by "slug".
+            if isinstance(self._get_queryset().first(), TreeNode):
+                instance = self._get_queryset().get(slug=self.form_data.get("slug"))
                 self.assertInstanceEqual(instance, self.form_data)
             else:
-                instance = self._get_queryset().last()
-                self.assertInstanceEqual(instance, self.form_data)
+                if hasattr(self.model, "last_updated"):
+                    instance = self._get_queryset().order_by("last_updated").last()
+                    self.assertInstanceEqual(instance, self.form_data)
+                else:
+                    instance = self._get_queryset().last()
+                    self.assertInstanceEqual(instance, self.form_data)
 
             if hasattr(self.model, "to_objectchange"):
                 # Verify ObjectChange creation
@@ -344,10 +362,16 @@ class ViewTestCases:
             }
             self.assertHttpStatus(self.client.post(**request), 302)
             self.assertEqual(initial_count + 1, self._get_queryset().count())
-            if hasattr(self.model, "last_updated"):
-                self.assertInstanceEqual(self._get_queryset().order_by("last_updated").last(), self.form_data)
+            # order_by() is no supported by django TreeNode,
+            # So we directly retrieve the instance by "slug".
+            if isinstance(self._get_queryset().first(), TreeNode):
+                instance = self._get_queryset().get(slug=self.form_data.get("slug"))
+                self.assertInstanceEqual(instance, self.form_data)
             else:
-                self.assertInstanceEqual(self._get_queryset().last(), self.form_data)
+                if hasattr(self.model, "last_updated"):
+                    self.assertInstanceEqual(self._get_queryset().order_by("last_updated").last(), self.form_data)
+                else:
+                    self.assertInstanceEqual(self._get_queryset().last(), self.form_data)
 
         def test_slug_autocreation(self):
             """Test that slug is autocreated through ORM."""
@@ -361,14 +385,18 @@ class ViewTestCases:
             """Ensure save method does not modify slug that is passed in."""
             # This really should go on a models test page, but we don't have test structures for models.
             if self.slug_source is not None:
+                new_slug_source_value = "kwyjibo"
+
                 obj = self.model.objects.get(**{self.slug_source: self.slug_test_object})
                 expected_slug = self.slugify_function(getattr(obj, self.slug_source))
                 # Update slug source field str
-                filter_ = self.slug_source + "__contains"
-                self.model.objects.filter(**{filter_: self.slug_test_object}).update(**{self.slug_source: "Test"})
+                filter_ = self.slug_source + "__exact"
+                self.model.objects.filter(**{filter_: self.slug_test_object}).update(
+                    **{self.slug_source: new_slug_source_value}
+                )
 
                 obj.refresh_from_db()
-                self.assertEqual(getattr(obj, self.slug_source), "Test")
+                self.assertEqual(getattr(obj, self.slug_source), new_slug_source_value)
                 self.assertEqual(obj.slug, expected_slug)
 
     class EditObjectViewTestCase(ModelViewTestCase):
@@ -462,8 +490,20 @@ class ViewTestCases:
         Delete a single instance.
         """
 
+        def get_deletable_object(self):
+            """
+            Get an instance that can be deleted.
+
+            For some models this may just be any random object, but when we have FKs with `on_delete=models.PROTECT`
+            (as is often the case) we need to find or create an instance that doesn't have such entanglements.
+            """
+            instance = get_deletable_objects(self.model, self._get_queryset()).first()
+            if instance is None:
+                self.fail("Couldn't find a single deletable object!")
+            return instance
+
         def test_delete_object_without_permission(self):
-            instance = self._get_queryset().first()
+            instance = self.get_deletable_object()
 
             # Try GET without permission
             with disable_warnings("django.request"):
@@ -479,7 +519,7 @@ class ViewTestCases:
 
         @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
         def test_delete_object_with_permission(self):
-            instance = self._get_queryset().first()
+            instance = self.get_deletable_object()
 
             # Assign model-level permission
             obj_perm = ObjectPermission(name="Test permission", actions=["delete"])
@@ -507,7 +547,8 @@ class ViewTestCases:
 
         @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
         def test_delete_object_with_constrained_permission(self):
-            instance1, instance2 = self._get_queryset().all()[:2]
+            instance1 = self.get_deletable_object()
+            instance2 = self._get_queryset().exclude(pk=instance1.pk)[0]
 
             # Assign object-level permission
             obj_perm = ObjectPermission(
@@ -535,12 +576,15 @@ class ViewTestCases:
                 self._get_queryset().get(pk=instance1.pk)
 
             # Try to delete a non-permitted object
+            # Note that in the case of tree models, deleting instance1 above may have cascade-deleted to instance2,
+            # so to be safe, we need to get another object instance that definitely exists:
+            instance3 = self._get_queryset().first()
             request = {
-                "path": self._get_url("delete", instance2),
+                "path": self._get_url("delete", instance3),
                 "data": post_data({"confirm": True}),
             }
             self.assertHttpStatus(self.client.post(**request), 404)
-            self.assertTrue(self._get_queryset().filter(pk=instance2.pk).exists())
+            self.assertTrue(self._get_queryset().filter(pk=instance3.pk).exists())
 
     class ListObjectsViewTestCase(ModelViewTestCase):
         """
@@ -552,12 +596,22 @@ class ViewTestCases:
         def get_filterset(self):
             return self.filterset or get_filterset_for_model(self.model)
 
+        # Helper methods to be overriden by special cases.
+        # See ConsoleConnectionsTestCase, InterfaceConnectionsTestCase and PowerConnectionsTestCase
+        def get_list_url(self):
+            return reverse(validated_viewname(self.model, "list"))
+
+        def get_title(self):
+            return bettertitle(self.model._meta.verbose_name_plural)
+
         @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
         def test_list_objects_anonymous(self):
             # Make the request as an unauthenticated user
             self.client.logout()
             response = self.client.get(self._get_url("list"))
             self.assertHttpStatus(response, 200)
+            response_body = response.content.decode(response.charset)
+            self.assertIn("/login/?next=" + self._get_url("list"), response_body, msg=response_body)
 
         @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
         def test_list_objects_filtered(self):
@@ -567,6 +621,9 @@ class ViewTestCases:
             content = extract_page_body(response.content.decode(response.charset))
             # TODO: it'd make test failures more readable if we strip the page headers/footers from the content
             if hasattr(self.model, "name"):
+                # TODO: https://github.com/nautobot/nautobot/issues/2580
+                #       This is fragile as we move toward more autogenerated test fixtures,
+                #       as "instance.name" may appear coincidentally in other page text if we're not careful.
                 self.assertIn(instance1.name, content, msg=content)
                 self.assertNotIn(instance2.name, content, msg=content)
             try:
@@ -578,20 +635,13 @@ class ViewTestCases:
         @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"], STRICT_FILTERING=True)
         def test_list_objects_unknown_filter_strict_filtering(self):
             """Verify that with STRICT_FILTERING, an unknown filter results in an error message and no matches."""
-            instance1, instance2 = self._get_queryset().all()[:2]
             response = self.client.get(f"{self._get_url('list')}?ice_cream_flavor=chocolate")
             self.assertHttpStatus(response, 200)
             content = extract_page_body(response.content.decode(response.charset))
             # TODO: it'd make test failures more readable if we strip the page headers/footers from the content
             self.assertIn("Unknown filter field", content, msg=content)
-            if hasattr(self.model, "name"):
-                self.assertNotIn(instance1.name, content, msg=content)
-                self.assertNotIn(instance2.name, content, msg=content)
-            try:
-                self.assertNotIn(self._get_url("view", instance=instance1), content, msg=content)
-                self.assertNotIn(self._get_url("view", instance=instance2), content, msg=content)
-            except NoReverseMatch:
-                pass
+            # There should be no table rows displayed except for the empty results row
+            self.assertIn(f"No {self.model._meta.verbose_name_plural} found", content, msg=content)
 
         @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"], STRICT_FILTERING=False)
         def test_list_objects_unknown_filter_no_strict_filtering(self):
@@ -599,10 +649,17 @@ class ViewTestCases:
             instance1, instance2 = self._get_queryset().all()[:2]
             with self.assertLogs("nautobot.utilities.filters") as cm:
                 response = self.client.get(f"{self._get_url('list')}?ice_cream_flavor=chocolate")
+            filterset = self.get_filterset()
+            if not filterset:
+                self.fail(
+                    f"Couldn't find filterset for model {self.model}. The FilterSet class is expected to be in the "
+                    "filters module within the application associated with the model and its name is expected to be "
+                    f"{self.model.__name__}FilterSet."
+                )
             self.assertEqual(
                 cm.output,
                 [
-                    f"WARNING:nautobot.utilities.filters:{self.get_filterset().__name__}: "
+                    f"WARNING:nautobot.utilities.filters:{filterset.__name__}: "
                     'Unknown filter field "ice_cream_flavor"',
                 ],
             )
@@ -610,7 +667,11 @@ class ViewTestCases:
             content = extract_page_body(response.content.decode(response.charset))
             # TODO: it'd make test failures more readable if we strip the page headers/footers from the content
             self.assertNotIn("Unknown filter field", content, msg=content)
+            self.assertNotIn(f"No {self.model._meta.verbose_name_plural} found", content, msg=content)
             if hasattr(self.model, "name"):
+                # TODO: https://github.com/nautobot/nautobot/issues/2580
+                #       This is fragile as we move toward more autogenerated test fixtures,
+                #       as "instance.name" may appear coincidentally in other page text if we're not careful.
                 self.assertIn(instance1.name, content, msg=content)
                 self.assertIn(instance2.name, content, msg=content)
             try:
@@ -624,7 +685,10 @@ class ViewTestCases:
 
             # Try GET without permission
             with disable_warnings("django.request"):
-                self.assertHttpStatus(self.client.get(self._get_url("list")), 403)
+                response = self.client.get(self._get_url("list"))
+                self.assertHttpStatus(response, 403)
+                response_body = response.content.decode(response.charset)
+                self.assertNotIn("/login/", response_body, msg=response_body)
 
         @override_settings(EXEMPT_VIEW_PERMISSIONS=[])
         def test_list_objects_with_permission(self):
@@ -636,7 +700,22 @@ class ViewTestCases:
             obj_perm.object_types.add(ContentType.objects.get_for_model(self.model))
 
             # Try GET with model-level permission
-            self.assertHttpStatus(self.client.get(self._get_url("list")), 200)
+            response = self.client.get(self._get_url("list"))
+            self.assertHttpStatus(response, 200)
+            response_body = response.content.decode(response.charset)
+
+            list_url = self.get_list_url()
+            title = self.get_title()
+
+            # Check if breadcrumb is rendered correctly
+            self.assertIn(
+                f'<a href="{list_url}">{title}</a>',
+                response_body,
+            )
+            # Check plugin banner is rendered correctly
+            self.assertIn(
+                f"<div>You are viewing a table of {self.model._meta.verbose_name_plural}</div>", response_body
+            )
 
             # Built-in CSV export
             if hasattr(self.model, "csv_headers"):
@@ -807,6 +886,28 @@ class ViewTestCases:
             self.assertHttpStatus(self.client.post(self._get_url("import"), data), 200)
             self.assertEqual(self._get_queryset().count(), initial_count + len(self.csv_data) - 1)
 
+        @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
+        def test_bulk_import_objects_with_permission_csv_file(self):
+            initial_count = self._get_queryset().count()
+            self.file_contents = bytes(self._get_csv_data(), "utf-8")
+            self.bulk_import_file = SimpleUploadedFile(name="bulk_import_data.csv", content=self.file_contents)
+            data = {
+                "csv_file": self.bulk_import_file,
+            }
+
+            # Assign model-level permission
+            obj_perm = ObjectPermission(name="Test permission", actions=["add"])
+            obj_perm.save()
+            obj_perm.users.add(self.user)
+            obj_perm.object_types.add(ContentType.objects.get_for_model(self.model))
+
+            # Try GET with model-level permission
+            self.assertHttpStatus(self.client.get(self._get_url("import")), 200)
+
+            # Test POST with permission
+            self.assertHttpStatus(self.client.post(self._get_url("import"), data), 200)
+            self.assertEqual(self._get_queryset().count(), initial_count + len(self.csv_data) - 1)
+
         @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
         def test_bulk_import_objects_with_constrained_permission(self):
             initial_count = self._get_queryset().count()
@@ -879,6 +980,33 @@ class ViewTestCases:
             for instance in self._get_queryset().filter(pk__in=pk_list):
                 self.assertInstanceEqual(instance, self.bulk_edit_data)
 
+        @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
+        def test_bulk_edit_form_contains_all_pks(self):
+            # We are testing the intermediary step of bulk_edit with pagination applied.
+            # i.e. "_all" passed in the form.
+            pk_list = self._get_queryset().values_list("pk", flat=True)
+            # We only pass in one pk to test the functionality of "_all"
+            # which should grab all instance pks regardless of "pk"
+            selected_data = {
+                "pk": pk_list[:1],
+                "_all": "on",
+            }
+            # Assign model-level permission
+            obj_perm = ObjectPermission(name="Test permission", actions=["change"])
+            obj_perm.save()
+            obj_perm.users.add(self.user)
+            obj_perm.object_types.add(ContentType.objects.get_for_model(self.model))
+
+            # Try POST with model-level permission
+            response = self.client.post(self._get_url("bulk_edit"), selected_data)
+            # Expect a 200 status cause we are only rendering the bulk edit table.
+            # after pressing Edit Selected button.
+            self.assertHttpStatus(response, 200)
+            response_body = extract_page_body(response.content.decode(response.charset))
+            # Check if all the pks are passed into the BulkEditForm/BulkUpdateForm
+            for pk in pk_list:
+                self.assertIn(f'<input type="hidden" name="pk" value="{pk}"', response_body)
+
         @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
         def test_bulk_edit_objects_with_constrained_permission(self):
             pk_list = list(self._get_queryset().values_list("pk", flat=True)[:3])
@@ -893,7 +1021,9 @@ class ViewTestCases:
             # Dynamically determine a constraint that will *not* be matched by the updated objects.
             attr_name = list(self.bulk_edit_data.keys())[0]
             field = self.model._meta.get_field(attr_name)
-            value = field.value_from_object(self._get_queryset().first())
+            value = field.value_from_object(
+                self._get_queryset().exclude(**{attr_name: self.bulk_edit_data[attr_name]}).first()
+            )
 
             # Assign constrained permission
             obj_perm = ObjectPermission(
@@ -923,9 +1053,18 @@ class ViewTestCases:
         Delete multiple instances.
         """
 
+        def get_deletable_object_pks(self):
+            """
+            Get a list of PKs corresponding to objects that can be safely bulk-deleted.
+
+            For some models this may just be any random objects, but when we have FKs with `on_delete=models.PROTECT`
+            (as is often the case) we need to find or create an instance that doesn't have such entanglements.
+            """
+            return get_deletable_objects(self.model, self._get_queryset()).values_list("pk", flat=True)[:3]
+
         @override_settings(EXEMPT_VIEW_PERMISSIONS=[])
         def test_bulk_delete_objects_without_permission(self):
-            pk_list = list(self._get_queryset().values_list("pk", flat=True)[:3])
+            pk_list = self.get_deletable_object_pks()
             data = {
                 "pk": pk_list,
                 "confirm": True,
@@ -938,7 +1077,8 @@ class ViewTestCases:
 
         @override_settings(EXEMPT_VIEW_PERMISSIONS=[])
         def test_bulk_delete_objects_with_permission(self):
-            pk_list = self._get_queryset().values_list("pk", flat=True)
+            pk_list = self.get_deletable_object_pks()
+            initial_count = self._get_queryset().count()
             data = {
                 "pk": pk_list,
                 "confirm": True,
@@ -953,12 +1093,39 @@ class ViewTestCases:
 
             # Try POST with model-level permission
             self.assertHttpStatus(self.client.post(self._get_url("bulk_delete"), data), 302)
-            self.assertEqual(self._get_queryset().count(), 0)
+            self.assertEqual(self._get_queryset().count(), initial_count - len(pk_list))
+
+        @override_settings(EXEMPT_VIEW_PERMISSIONS=[])
+        def test_bulk_delete_form_contains_all_pks(self):
+            # We are testing the intermediary step of bulk_delete with pagination applied.
+            # i.e. "_all" passed in the form.
+            pk_list = self._get_queryset().values_list("pk", flat=True)
+            # We only pass in one pk to test the functionality of "_all"
+            # which should grab all instance pks regardless of "pks".
+            selected_data = {
+                "pk": pk_list[:1],
+                "confirm": True,
+                "_all": "on",
+            }
+
+            # Assign unconstrained permission
+            obj_perm = ObjectPermission(name="Test permission", actions=["delete"])
+            obj_perm.save()
+            obj_perm.users.add(self.user)
+            obj_perm.object_types.add(ContentType.objects.get_for_model(self.model))
+
+            # Try POST with the selected data first. Emulating selecting all -> pressing Delete Selected button.
+            response = self.client.post(self._get_url("bulk_delete"), selected_data)
+            self.assertHttpStatus(response, 200)
+            response_body = extract_page_body(response.content.decode(response.charset))
+            # Check if all the pks are passed into the BulkDeleteForm/BulkDestroyForm
+            for pk in pk_list:
+                self.assertIn(f'<input type="hidden" name="pk" value="{pk}"', response_body)
 
         @override_settings(EXEMPT_VIEW_PERMISSIONS=[])
         def test_bulk_delete_objects_with_constrained_permission(self):
+            pk_list = self.get_deletable_object_pks()
             initial_count = self._get_queryset().count()
-            pk_list = self._get_queryset().values_list("pk", flat=True)
             data = {
                 "pk": pk_list,
                 "confirm": True,
@@ -985,7 +1152,7 @@ class ViewTestCases:
 
             # Bulk delete permitted objects
             self.assertHttpStatus(self.client.post(self._get_url("bulk_delete"), data), 302)
-            self.assertEqual(self._get_queryset().count(), 0)
+            self.assertEqual(self._get_queryset().count(), initial_count - len(pk_list))
 
     class BulkRenameObjectsViewTestCase(ModelViewTestCase):
         """
