@@ -4,8 +4,9 @@ from django.test import TestCase
 
 from nautobot.dcim.models import Location, LocationType, Site
 from nautobot.extras.models import Status
+from nautobot.ipam.models import VLAN
 from nautobot.tenancy.models import Tenant
-from nautobot.virtualization.models import VirtualMachine, ClusterType, Cluster
+from nautobot.virtualization.models import VirtualMachine, ClusterType, Cluster, VMInterface
 
 
 class ClusterTestCase(TestCase):
@@ -69,3 +70,20 @@ class VirtualMachineTestCase(TestCase):
         # Two VMs assigned to the same Cluster and different Tenants should pass validation
         vm2.full_clean()
         vm2.save()
+
+
+class VMInterfaceTestCase(TestCase):
+    def setUp(self):
+        site = Site.objects.create(name="Site-1", slug="site-1")
+        self.vlan = VLAN.objects.create(name="VLAN 1", vid=100, site=site)
+        clustertype = ClusterType.objects.create(name="Test Cluster Type 1", slug="test-cluster-type-1")
+        cluster = Cluster.objects.create(name="Test Cluster 1", type=clustertype)
+        self.virtualmachine = VirtualMachine.objects.create(cluster=cluster, name="Test VM 1")
+
+    def test_tagged_vlan_raise_error_if_mode_not_set_to_tagged(self):
+        interface = VMInterface.objects.create(virtual_machine=self.virtualmachine, name="Interface 1")
+        with self.assertRaises(ValidationError) as err:
+            interface.tagged_vlans.add(self.vlan)
+        self.assertEqual(
+            err.exception.message_dict["tagged_vlans"][0], "Mode must be set to tagged when specifying tagged_vlans"
+        )
